/****************************************************************************
*
*	  Module Filename:  Interaction_Type.cpp
*	  Module Name:      Interaction_Type
*	  Description:      An Interaction_Type is a possible
*                     type of chemical interaction.
*	  Author:           Chris Gordon-Smith
*
*     Copyright (C) 2003, 2005, 2006, 2008 Chris Gordon-Smith
*     Email: See http://www.simsoup.info/
*
*     This program is free software; you can redistribute it and/or modify it
*     under the terms of the GNU General Public License Version 2, as published 
*     by the Free Software Foundation.
*
*     This program is distributed in the hope that it will be useful, but
*     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
*     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
*     for more details.
*
*     You should have received a copy of the GNU General Public License along
*     with this program; if not, write to the Free Software Foundation, Inc.,
*     59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*     See the file Licence.txt for a copy of the GNU General Public License.
*
*	  Amendment History:-
*
*		Version	   Date	   			Update Description
*		=======    ===========		==================
*
*
*
*
****************************************************************************/
//  Comment out to disallow tracing this file
//#define  ENABLE_TRACE_FILE  

#include  "globldef.h"
#include  "Library_Headers.h"

namespace DN {
#include "list.hpp"        // Special purpose - Non STL
}

//--------------  Chemistry Subsystem  --------------------------------------------------------------------------
#include  "Interaction_Type.h"
#include  "Chemistry.h"
#include  "Molecule_Type.h"
//---------------------------------------------------------------------------------------------------------------

//---------------- Reactor Subsystem ----------------------------------------------------------------------------
//#include  "Reactor.h"
#include  "IntType_Notify_API.h"
//----------------------------------------------------------------------------------------------------------

#include  <gsl/gsl_rng.h>    // GNU Scientific library
//---------------- Utilities Subsystem ----------------------------------------------------------------------------
#include  "Random_Sequence.h"
//----------------------------------------------------------------------------------------------------------

//  Initialise static data

    int Interaction_Type::Next_Interaction_Type_Identifier = 0;

//  Parameters for use with the 'Random' Rate Constant Model

    double Construction::Max_Rand_Rate_Constant = 0.0;
    double Fission::Max_Rand_Rate_Constant = 0.0;
    double Transformation::Max_Rand_Rate_Constant = 0.0;

//  Parameters for use with the Arrenhius Rate Constant Model

    double Interaction_Type::Max_Activated_Complex_Energy = 20.0;

//----------------------------------------------------------------------------------------------------------

void  Interaction_Type::Clear_Static()

//  Cleear the static data for the Interaction_Type class

{
    Next_Interaction_Type_Identifier = 0;
}
//----------------------------------------------------------------------------------------------------------------
void    Construction::Add_To_Chemistry(int Reactant1_Index,
                                       int Product1_Index,
                                       int Reactant2_Index)

//  Add the Constrution to the Chemistry and link to Molecule_Types

{
    TRACE("Construction::Add_To_Chemistry(int Reactant1_Index, int Product1_Index, int Reactant2_Index)");

    Assert(Validate_Unique_Integers(Reactant1_Index, Product1_Index, Reactant2_Index),
            "Construction::Add_To_Chemistry - Duplicate Molecule Types");

//  First call the Add_To_Chemistry function for the base class (Interaction_Type)
//  to do the processing common to all Interaction_Types

    Interaction_Type*   Interaction_Type__Ptr;
    Interaction_Type__Ptr = this;
    Interaction_Type__Ptr->Add_To_Chemistry(Reactant1_Index, Product1_Index);

//  Link the Construction to the Reactant 2
                                            
    Molecule_Type__Reactant2_Ptr =  static_cast<Molecule_Type*>
        (Chemistry::Chemistry__Ptr->TList__Of_Molecule_Types_Ptr->Get_Item(Reactant2_Index));

    Assert (Molecule_Type__Reactant2_Ptr not_eq 0,"Construction::Add_To_Chemistry - Invalid Reacxtant 2");
                                     
//  Add the Construction to the list of Interaction_Types for the Reactant2 Molecule_Type

    Molecule_Type__Reactant2_Ptr->List_Of_Int_Types_With_MolType_As_Input.Add(this);

//  Inform the Reactor of the new Interaction_Type

//    Reactor::Get_Reactor__Ptr()->Notify_New_Int_Type(this);  
    IntType_Notify_API::Notify_New_IntType(this);  
}
//------------------------------------------------------------------------------

double Construction::Calculate_Reactor_Interaction_Rate
        (const Calculate_Reactor_Interaction_Rates_Args* const Args) const
        
//  Calculate the Interaction Rate for this Interaction_Type using the Numbers of Molecules
//  of the Reactants and Product in the Reactor and the forward and reverese Reaction Rate 
//  constants for the Construction.

//  For the forward reaction, it is assumed that each Molecule has a single collision in unit time. 
//  The Forward Interaction_Rate is then the number of Molecules of one Reactant, times the probability that
//  each of these Molecules collides with a Molecule of the other Reactant, times the rate constant

//  The Reverse Interaction Rate is calculated in the same way as the fprward rate for a Fission.Inputs

//  The overall rate is either the Forward Rate minus the Reverse Rate, or zero if the Reverse Rate is 
//  greater than the Reverse Rate

{

//  Reactant2_Num_Mols argument is only set in the case of Interaction_Types with a Reactant2.
//  Check it was set correctly

    Assert(Args->Reactant2_Num_Mols >= 0,
            "Construction::Calculate_Reactor_Interaction_Rate - Invalid Reactant2_Num_Mols Argument");

//  Product2_Num_Mols argument is only set in the case of Interaction_Types with a Product2.
//  Check it was set correctly

    Assert(Args->Product2_Num_Mols < 0,
            "Construction::Calculate_Reactor_Interaction_Rate - Invalid Product2_Num_Mols Argument");

//  Convert arguments to double to ensure no integer overflow

    double R1_Num_Mols = static_cast<double> (Args->Reactant1_Num_Mols);
    double R2_Num_Mols = static_cast<double> (Args->Reactant2_Num_Mols);
    double P1_Num_Mols = static_cast<double> (Args->Product1_Num_Mols);

//  Make sure we don't have too many Molecules of either Reactant (forward reaction)
//  or of the Product (reverse reaction)

    Assert(R1_Num_Mols <= 100000000, 
        "Construction::Calculate_Reactor_Interaction_Rate: Number of Molecules of Reactant 1 exceeds 100000000.");
    Assert(R2_Num_Mols <= 100000000, 
        "Construction::Calculate_Reactor_Interaction_Rate: Number of Molecules of Reactant 2 exceeds 100000000.");
    Assert(P1_Num_Mols <= 100000000, 
        "Construction::Calculate_Reactor_Interaction_Rate: Number of Molecules of Product 1 exceeds 100000000.");
//  Product2_Num_Mols shoud be -1 in the case of a Construction. Check this is working.
    Assert(Args->Product2_Num_Mols  == -1,
            "Construction::Calculate_Reactor_Interaction_Rate - Invalid Argument");

//  Now calculate the Interaction Rate

    double Forward_Rate = (Forward_Rate_Constant * (R1_Num_Mols * R2_Num_Mols))/ Args->Reactor_Volume;
    double Reverse_Rate = Reverse_Rate_Constant * P1_Num_Mols;

//  ================================================
    return (max((Forward_Rate - Reverse_Rate),0.0));
//  ================================================
}
//----------------------------------------------------------------------------------------------------------
bool Construction::Check_If_Construction_Exists(int Reactant1_Index,
                                                int Product1_Index,
                                                int Reactant2_Index)

//  Check whether a Construction with the specified characteristics

//  already exists in the Chemistry
//  Do this by searching the list of Interaction_Types for Reactant 1.

{
    TRACE("Construction::Check_If_Construction_Exists");
    Assert(Reactant1_Index >= 0, "Construction::Check_If_Construction_Exists - Invalid Reactant 1");

    Molecule_Type*  Molecule_Type__Reactant1_Ptr;

    Interaction_Type*   Interaction_Type__Ptr;

 
    int         Interaction_Type_Identifier;
    Interaction_Type_Enum  Interaction_Type_Enumr;
    int         Reactant1_Identifier;
    int         Reactant2_Identifier;
    int         Product1_Identifier;
    int         Product2_Identifier;
    int         Catalyst_Identifier;

//  Construction cannot exist if Reactant 1 does not exist
    if (Chemistry::Get_Num_MolTypes() < (Reactant1_Index + 1)) return false;

    Molecule_Type__Reactant1_Ptr
        = (Molecule_Type*) Chemistry::Chemistry__Ptr
            ->TList__Of_Molecule_Types_Ptr
                ->Get_Item(Reactant1_Index);

    for (int Continue_Flag = Molecule_Type__Reactant1_Ptr->List_Of_Int_Types_With_MolType_As_Input.Iter_To_First();
         Continue_Flag == true;
         Continue_Flag = Molecule_Type__Reactant1_Ptr->List_Of_Int_Types_With_MolType_As_Input.Iter_To_Next())
    BEGIN

        Interaction_Type__Ptr =
            Molecule_Type__Reactant1_Ptr->List_Of_Int_Types_With_MolType_As_Input.Get_Current_Item();


        Interaction_Type__Ptr->Get_Int_Type_Characteristics

                            (Interaction_Type_Identifier,
                             Interaction_Type_Enumr,
                             Reactant1_Identifier,

                             Reactant2_Identifier,
                             Product1_Identifier,
                             Product2_Identifier,
                             Catalyst_Identifier);


        if ((Interaction_Type_Enumr == Interaction_Type_Enumr__Constr)

                and
                (((Reactant1_Index == Reactant1_Identifier)

                    and
                (Product1_Index == Product1_Identifier)
                    and
                (Reactant2_Index == Reactant2_Identifier))
                            or
                ((Reactant2_Index == Reactant1_Identifier)
                    and
                (Product1_Index == Product1_Identifier)
                    and
                (Reactant1_Index == Reactant2_Identifier)))
           )
        THEN
            return(true);
        END
    END
    return(false);
}
//------------------------------------------------------------------------------

//Construction::Construction()
//{


//  Setup unique characteristics for this Construction
//  and then add it to the Chemistry

//     int Reactant1_Index;
//     int Product1_Index;
//     int Reactant2_Index;
// 
//     if (not Specify_Unique_Construction(true, Reactant1_Index,   // Chosse Reactan1 at random
//                                         true, Product1_Index,    // and Product
//                                         true, Reactant2_Index,   // and Reactant 2
// 					                    true, Activated_Complex_Energy))
//     THEN
// //      Couldn't find a new Construction
// 
//         throw Create_Error(string("Insufficient possibilities for new Constructions."));
//     ELSE
// 
// //      Add this Construction to the Chemistry
// 
//         Add_To_Chemistry(Reactant1_Index, Product1_Index, Reactant2_Index);
// 
//         Determine_Rate_Constants();  // Do it after Add_To_Chemistry because MolType pointers needed
//     ENDIF
//}
//----------------------------------------------------------------------------------------------------------
Construction::Construction(int R1_Id, int R2_Id, int P_Id,
                           double   Actv_Cplx_Energy)

{
    TRACE("Construction::Construction(int R1_Id, int R2_Id, int P_Id, double   Actv_Cplx_Energy");

//  Constructor - Base new Construction on specified Molecule_Types

    Molecule_Type__Reactant1_Ptr  = Chemistry::Get_MolType_Pt(R1_Id);
    Molecule_Type__Reactant2_Ptr  = Chemistry::Get_MolType_Pt(R2_Id);
    Molecule_Type__Product1_Ptr  = Chemistry::Get_MolType_Pt(P_Id);

    Assert (Molecule_Type__Reactant1_Ptr != 0, "Construction::Construction - Reactant1_Ptr = 0");
    Assert (Molecule_Type__Reactant2_Ptr != 0, "Construction::Construction - Reactant2_Ptr = 0");
    Assert (Molecule_Type__Product1_Ptr != 0, "Construction::Construction - Product1_Ptr = 0");
    Assert (Actv_Cplx_Energy >= 0, "Construction::Construction - Invalid Activated_Complex_Energy = ");
    
    Assert (Molecule_Type__Reactant1_Ptr  not_eq Molecule_Type__Reactant2_Ptr  , "Construction::Construction - Reactant1 = Reactant2");
    Assert (Molecule_Type__Reactant2_Ptr  not_eq Molecule_Type__Product1_Ptr  , "Construction::Construction - Product1 = Reactant2");
    Assert (Molecule_Type__Product1_Ptr not_eq  Molecule_Type__Reactant1_Ptr  , "Construction::Construction - Reactant1 = Product1");

//  Activated_Complex_Energy must be higher than the Potential Energy of Reactants and of Product

    Assert (Actv_Cplx_Energy > (Molecule_Type__Reactant1_Ptr ->Get_Potential_Energy() 
                                + Molecule_Type__Reactant2_Ptr  ->Get_Potential_Energy()),
            "Construction::Construction - Activated Complex Energy is less than Energy of Reactants");
    Assert (Actv_Cplx_Energy > (Molecule_Type__Product1_Ptr ->Get_Potential_Energy()),
            "Construction::Construction - Activated Complex Energy is less than Energy of Product");
 
    Activated_Complex_Energy = Actv_Cplx_Energy;

    Determine_Rate_Constants();

//  Add this Construction to the Chemistry

    Add_To_Chemistry(Molecule_Type__Reactant1_Ptr->Get_Identifier(),
                     Molecule_Type__Product1_Ptr->Get_Identifier(),
                     Molecule_Type__Reactant2_Ptr->Get_Identifier());
}

//----------------------------------------------------------------------------------------------------------
/*
Interaction_Type*  Construction::Copy_With_Variation(int Old_MolType_Index, int New_MolType_Index)

//  Make a modified copy of this Construction, such that if the existing Interaction_Type
//  has a reference to 'Old MolType', the new Construction is produced with the reference
//  replaced by a reference to 'New Mol Type'.


//  Make sure that all three Molecule_Types are different

{

    Molecule_Type*  Mol_Type_Reactant1_Ptr;
    Molecule_Type*  Mol_Type_Reactant2_Ptr;
    Molecule_Type*  Mol_Type_Product1_Ptr;

    if (Molecule_Type__Reactant1_Ptr->Get_Identifier() == Old_MolType_Index)
    THEN
        Mol_Type_Reactant1_Ptr = Chemistry::Get_Chemistry__Ptr()->Get_Molecule_Type_Ptr(New_MolType_Index);
    ELSE
        Mol_Type_Reactant1_Ptr = Molecule_Type__Reactant1_Ptr;
    ENDIF

    if (Molecule_Type__Reactant2_Ptr->Get_Identifier() == Old_MolType_Index)
    THEN
        Mol_Type_Reactant2_Ptr = Chemistry::Get_Chemistry__Ptr()->Get_Molecule_Type_Ptr(New_MolType_Index);
    ELSE

        Mol_Type_Reactant2_Ptr = Molecule_Type__Reactant2_Ptr;
    ENDIF

    if (Molecule_Type__Product1_Ptr->Get_Identifier() == Old_MolType_Index)
    THEN
        Mol_Type_Product1_Ptr = Chemistry::Get_Chemistry__Ptr()->Get_Molecule_Type_Ptr(New_MolType_Index);
    ELSE
        Mol_Type_Product1_Ptr = Molecule_Type__Product1_Ptr;
    ENDIF

//  Make sure that all Molecule Types are different. If OK, then create Constructio

    int  Reactant1_Index =   Mol_Type_Reactant1_Ptr->Get_Identifier();
    int  Reactant2_Index =   Mol_Type_Reactant2_Ptr->Get_Identifier();
    int  Product1_Index =   Mol_Type_Product1_Ptr->Get_Identifier();
    
    if (!Validate_Construction_Parameters(Reactant1_Index,
                                          Reactant2_Index,
                                          Product1_Index))
    THEN
        return  0;
    ELSE               
        Interaction_Type*  New_Int_Ptr;
        CREATE_OBJECT(Construction, New_Int_Ptr 
            = new  Construction(Mol_Type_Reactant1_Ptr,
                                Mol_Type_Reactant2_Ptr,
                                Mol_Type_Product1_Ptr,
//                                Rate_Constant,        // Reaction Rate is Unchanged
								Activation_Energy));  // Activation Energyis unchanged
        return  New_Int_Ptr;
    ENDIF
}  */
//-----------------------------------------------------------------------------------------------------------------

void Construction::Determine_Rate_Constants()

//  Set the forward and reverse Rate Constants for the Construction.
//  Set them at random or according to Arrenhius rules depending on the argument

{	
    TRACE("Construction::Determine_Rate_Constants");
    if (Chemistry::Get_Rate_Constant_Model() == Chemistry::Arrenhius)
    THEN
        double Forward_Activation_Energy = Get_Forward_Activation_Energy();
        double Reverse_Activation_Energy = Get_Reverse_Activation_Energy();
//      ========================================================
        Forward_Rate_Constant = exp(-Forward_Activation_Energy);
        Reverse_Rate_Constant = exp(-Reverse_Activation_Energy);
//      ========================================================

    ELSEIF (Chemistry::Get_Rate_Constant_Model() == Chemistry::Random)
    THEN
        Forward_Rate_Constant = Random_Sequence::Get_Ptr(Random_Sequence::One)->Uniform(Max_Rand_Rate_Constant);
        Reverse_Rate_Constant = Random_Sequence::Get_Ptr(Random_Sequence::One)->Uniform(Max_Rand_Rate_Constant);
    ELSE
        Assert (true == false, "Construction::Determine_Rate_Constants() - Invalid Rate Constant Model");
    ENDIF
    Assert ((Forward_Rate_Constant >= 0) and (Reverse_Rate_Constant >= 0), 
           "Construction::Determine_Rate_Constant() - Invalid Rate_Constant");
}
//----------------------------------------------------------------------------------------------------------------

double Construction::Get_Forward_Activation_Energy() const

//  Return the Activation Energy for the forward Construction

{
    double R1_PE = Get_Reactant1_MolType_Ptr()->Get_Potential_Energy();
    double R2_PE = Get_Reactant2_MolType_Ptr()->Get_Potential_Energy();
    double Forward_Activation_Energy = Activated_Complex_Energy - (R1_PE + R2_PE);
    return Forward_Activation_Energy;
}
//----------------------------------------------------------------------------------------------------------
/*
void Construction::Draw(Network_Painter_Prot* const Network_Painter_Prot__Ptr,
                        double Interaction_Rate,
                        int Conn_Selection_State) const


//  Draw the Construction using the specified Network_Painter

{

//  Call the Network_Painter to display the Construction
//  Use the Molecule_Type Identifiers as the Node Indexes

    Network_Painter_Prot__Ptr->Draw_Construction(
                                                Molecule_Type__Reactant1_Ptr->Get_Identifier(),
                                                Molecule_Type__Reactant2_Ptr->Get_Identifier(),
                                                Molecule_Type__Product1_Ptr->Get_Identifier(),
                                                Interaction_Rate,
                                                Conn_Selection_State,
                                                true); // Draw for first time, so create a Network Display Item
                                                       // in case need to redraw;
} */
//----------------------------------------------------------------------------------------------------------
Interaction_Type_Enum  Construction::Get_Category() const


//  Return the category of Interaction_Type

{
    TRACE("Construction::Get_Category()");
    return(Interaction_Type_Enumr__Constr);
}
//----------------------------------------------------------------------------------------------------------------

string  Construction::Get_Category_As_String() const

//  Return the category of Interaction_Type

{
    return string("Construction");
}
//----------------------------------------------------------------------------------------------------------------

double  Construction::Get_Heat_Release() const

//  Return the amount of heat released by one Construction.
//  For an endothermic Construction, the amount will be negative

{
    return (Get_Reactant1_MolType_Ptr()->Get_Potential_Energy() 
            + Get_Reactant2_MolType_Ptr()->Get_Potential_Energy()
            - Get_Product1_MolType_Ptr()->Get_Potential_Energy());
}
//----------------------------------------------------------------------------------------------------------

void    Construction::Get_Int_Type_Characteristics
                                           (int& Interaction_Type_Identifier,
                                            Interaction_Type_Enum&  Interaction_Type_Enumr,
                                            int& Reactant1_Identifier,
                                            int& Reactant2_Identifier,
                                            int& Product1_Identifier,
                                            int& Product2_Identifier,
                                            int& Catalyst_Identifier) const

//  Return the characteristics of the Construction

{
    Product2_Identifier = 0;
    Catalyst_Identifier = 0;
    Interaction_Type_Identifier = Identifier;
    Interaction_Type_Enumr = Interaction_Type_Enumr__Constr;
    Reactant1_Identifier = Molecule_Type__Reactant1_Ptr->Get_Identifier();
    Reactant2_Identifier = Molecule_Type__Reactant2_Ptr->Get_Identifier();
    Product1_Identifier =  Molecule_Type__Product1_Ptr->Get_Identifier();
}

//------------------------------------------------------------------------------
void Construction::Get_Int_Type_Inputs_No_Dups(VCSList* TList__Of_Interaction_Inputs_Ptr) const

//  Put the Molecule_Type pointers for the Interaction_Type's inputs
//  (Reactants and Catalysts) in the TList. Don't include the
//  same Molecule_Type more than once. 

{
    Assert((Molecule_Type__Reactant1_Ptr not_eq 0) and (Molecule_Type__Reactant2_Ptr not_eq 0),
            "Construction::Get_Int_Type_Inputs_No_Dups - Invalid Reactant1 or Reactant2");
    TList__Of_Interaction_Inputs_Ptr->Clear();
    TList__Of_Interaction_Inputs_Ptr->Add(Molecule_Type__Reactant1_Ptr);
    if (Molecule_Type__Reactant1_Ptr != Molecule_Type__Reactant2_Ptr)
    THEN
        TList__Of_Interaction_Inputs_Ptr->Add(Molecule_Type__Reactant2_Ptr);
    ENDIF
}
//----------------------------------------------------------------------------------------------------------
void Construction::Get_Int_Type_Products_With_Dups(VCSList* TList__Of_Int_Type_Products_Ptr) const

//  Put the Molecule_Type pointer for the Interaction_Type's Product
//  in the TList.

{
    TList__Of_Int_Type_Products_Ptr->Clear();
    TList__Of_Int_Type_Products_Ptr->Add(Molecule_Type__Product1_Ptr);
}

//----------------------------------------------------------------------------------------------------------
void Construction::Get_Int_Type_Reactants_With_Dups(VCSList* TList__Of_Int_Type_Reactants_Ptr) const

//  Put the Molecule_Type pointers for the Interaction_Type's Reactants
//  in the TList. If the same Reactant occurs twive then include it in the list twice.

{
//    TList__Of_Int_Type_Reactants_Ptr->Count=0;

    TList__Of_Int_Type_Reactants_Ptr->Clear();
    TList__Of_Int_Type_Reactants_Ptr->Add(Molecule_Type__Reactant1_Ptr);
    TList__Of_Int_Type_Reactants_Ptr->Add(Molecule_Type__Reactant2_Ptr);
}
//----------------------------------------------------------------------------------------------------------------


double Construction::Get_Reverse_Activation_Energy() const

//  Return the Activation Energy for the reverse Construction

{
    double P1_PE = Get_Product1_MolType_Ptr()->Get_Potential_Energy();
    double Reverse_Activation_Energy = Activated_Complex_Energy - P1_PE;
    return Reverse_Activation_Energy;
}
//----------------------------------------------------------------------------------------------------------
void    Construction::Remove_From_Chemistry()



//  Remove this Construction from the Chemistry by unlinking it from its Molecule_Types
//  and marking it as logically deleted.  Don't destroy the object; this will
//  enable other objects (eg in the Reactor) to continue to refer to it.

{

//  First call the Remove_From_Chemistry function for the base class (Interaction_Type)
//  to do the processing common to all Interaction_Types

    Interaction_Type::Remove_From_Chemistry();

//  Remove the Construction from the list of Interaction_Types for the
//  Reactant2 Molecule_Type, provided it was not removed earlier

    if (Molecule_Type__Reactant2_Ptr != Molecule_Type__Reactant1_Ptr)
    THEN
        Molecule_Type__Reactant2_Ptr->List_Of_Int_Types_With_MolType_As_Input.Remove(this);
    ENDIF
}
//----------------------------------------------------------------------------------------------------------

void Construction::Set_Max_Rand_Rate_Constant(double Max_Rate)

//  Set the maximum rate constant for Constructions setup with the Random Rate Constant model

{
    Max_Rand_Rate_Constant = Max_Rate;
}
//----------------------------------------------------------------------------------------------------------------

bool Construction::Validate_Construction_Parameters(int Reactant1_Index,
                                                   int Product1_Index,
                                                   int Reactant2_Index,
                                                   double Actv_Cplx_Energy,
                                                   bool Rep_Fail_Reas_Fl,
                                                   string*& Fail_Tx_Pt)

//  Construction Parameters are invalid if two Mol Types are equal, or
//  if another construction with the same Molecule Types already exists
//  Also check that Mass of Product = Mass of Reactants.
//  And check that each Molecule_Type exists
//  In addition Activated Complex Energy must be higher than Reactants and higher than Product,
//  and the potential energy of the Reactants must be higher than that of the Product (so that
//  the forward rection is favoured).

{
    TRACE("Construction::Validate_Construction_Parameters");

    int Num_MolTypes = Chemistry::Get_Num_MolTypes();

    if((Reactant1_Index == Reactant2_Index)
                ||    
       (Reactant2_Index == Product1_Index)
                ||
       (Product1_Index == Reactant1_Index)
       )
    THEN
        if(Rep_Fail_Reas_Fl)
        THEN
            CREATE_OBJECT(string, Fail_Tx_Pt = new string(" Equal Molecule Types specified."));
        ENDIF
        return false;
        
    ELSEIF (Check_If_Construction_Exists(Reactant1_Index, Product1_Index, Reactant2_Index))
    THEN
        if(Rep_Fail_Reas_Fl)
        THEN
            CREATE_OBJECT(string, Fail_Tx_Pt = new string(" Construction already exists."));
        ENDIF
        return false;
    ELSEIF (Reactant1_Index >= Num_MolTypes
                    or
            Reactant1_Index < 0
                    or
            Reactant2_Index >= Num_MolTypes
                    or
            Reactant2_Index < 0
                    or
            Product1_Index >= Num_MolTypes
                    or
            Product1_Index < 0)
    THEN
        if(Rep_Fail_Reas_Fl)
        THEN
            CREATE_OBJECT(string, Fail_Tx_Pt = new string(" One of the specified Molecule Types is invalid or does not exist"));
        ENDIF
        return false;
    ELSE
        int Reactant1_Mass =  Chemistry::Get_MolType_Pt(Reactant1_Index)->Get_Mass();
        int Reactant2_Mass =  Chemistry::Get_MolType_Pt(Reactant2_Index)->Get_Mass();
        int Product1_Mass =  Chemistry::Get_MolType_Pt(Product1_Index)->Get_Mass();
        
        if ((Reactant1_Mass + Reactant2_Mass) not_eq Product1_Mass)
        THEN
            if(Rep_Fail_Reas_Fl)
            THEN
                CREATE_OBJECT(string, Fail_Tx_Pt = new string(" Product mass is not the same as the mass of the Reactants."));
            ENDIF
            return false;
        ELSE   //  Validate the Activated Complex Energy and the Potential Energies
            double R1_PE = Chemistry::Get_MolType_Pt(Reactant1_Index)->Get_Potential_Energy();
            double R2_PE = Chemistry::Get_MolType_Pt(Reactant2_Index)->Get_Potential_Energy();
            double P1_PE = Chemistry::Get_MolType_Pt(Product1_Index)->Get_Potential_Energy();
            if ((R1_PE + R2_PE) >= Actv_Cplx_Energy)
            THEN
                if(Rep_Fail_Reas_Fl)
                THEN
                    CREATE_OBJECT(string, 
                        Fail_Tx_Pt = new string (" Activated Complex Energy is less than the sum of Reactant Potential Energies."));
                ENDIF
                return false;
            ELSEIF(P1_PE >= Actv_Cplx_Energy)
            THEN
                if(Rep_Fail_Reas_Fl)
                THEN
                    CREATE_OBJECT(string, 
                        Fail_Tx_Pt = new string (" Activated Complex Energy is less than that of the Product."));
                ENDIF
                return false;
            ELSEIF(P1_PE >= (R1_PE + R2_PE))
            THEN
                if(Rep_Fail_Reas_Fl)
                THEN
                    CREATE_OBJECT(string, 
                        Fail_Tx_Pt = new string (" Potential Energy of Reactants is less than that of the Product."));
                ENDIF
                return false;
            ELSE
                return true;
            ENDIF
        ENDIF
    ENDIF
}                                                   

//----------------------------------------------------------------------------------------------------------
bool Construction::Specify_Unique_Construction(bool R1_Random, int& Reactant1_Index,
                                                bool P_Random, int& Product1_Index,
                                                bool R2_Random, int& Reactant2_Index,
					                            bool ACE_Rnd_Fl, double& Actvtd_Cplex_Enrgy,
                                                string*& Fail_Tx_Pt)

//   Generate characteristics for a unique
//   Construction that does not already exist in the Chemistry
//   R1_Random etc are set false if the value of the corresponding arguments specifies
//   a reactant / product that should not be changed. If the value is true then a
//   reactant / product must be chosen at random.

{
    TRACE("Construction::Specify_Unique_Construction");

    int Max_Attempts;
    if (R1_Random or P_Random or R2_Random or ACE_Rnd_Fl)
    THEN
        Max_Attempts = 1000;
    ELSE
        Max_Attempts = 1;  // No point in multiple attempts if everything is fixed
    ENDIF

    int     Loop_Counter = 0;
    do
    BEGIN

        Loop_Counter++;

//      Keep trying until a Construction that does not exist is found, or
//      return false if can't find one. Uniqueness is determined on the basis if
//      Inputs and Products only (not Rate_Constant)

        if (Loop_Counter > Max_Attempts)
        THEN
            if (Max_Attempts > 1)
            THEN
                Assert(Fail_Tx_Pt not_eq 0, "Construction::Specify_Unique_Construction- Invalid Fail_Tx_Pt");
               (*Fail_Tx_Pt) = string(" Insufficient feasible Constructions. Reason for failure of last attempt: ")
                              + (*Fail_Tx_Pt);
            ENDIF
            return false;
        ENDIF
        if (R1_Random == true)
        THEN
	       Reactant1_Index = Random_Sequence::Get_Ptr(Random_Sequence::One)
                ->Uniform(Chemistry::Chemistry__Ptr->Number_Of_Molecule_Types);
        ELSE
//          Keep the specified Reactant 1
        ENDIF

        if (P_Random == true)
        THEN
            Product1_Index = Random_Sequence::Get_Ptr(Random_Sequence::One)
                ->Uniform(Chemistry::Chemistry__Ptr->Number_Of_Molecule_Types);
        ELSE
//          Keep the specified Product
        ENDIF
        if (R2_Random == true)
        THEN
            Reactant2_Index = Random_Sequence::Get_Ptr(Random_Sequence::One)
                ->Uniform(Chemistry::Chemistry__Ptr->Number_Of_Molecule_Types);
        ELSE
//          Keep the specified Reactant 2
        ENDIF

//      Determine the Activated Complex Energy. It must be greater than energy of Reactants and of Product

        if (ACE_Rnd_Fl)
        THEN
            Actvtd_Cplex_Enrgy = Random_Sequence::Get_Ptr(Random_Sequence::One)
                ->Uniform(Get_Max_Activated_Complex_Energy()); 
        ELSE
//          Keep the specified value
        ENDIF
    END
    while ((Validate_Construction_Parameters(Reactant1_Index, 
                                             Product1_Index, 
                                             Reactant2_Index,
                                             Actvtd_Cplex_Enrgy,
                                             Loop_Counter == Max_Attempts,  // Report Fail Reason
                                             Fail_Tx_Pt)                    // for last attempt only


           == false));
    return true;
}
//----------------------------------------------------------------------------------------------------------
void    Fission::Add_To_Chemistry(int Reactant1_Index,
                                  int Product1_Index,
                                  int Product2_Index)

//  Add the Fission to the Chemistry and link to Molecule_Types

{
    TRACE("Fission::Add_To_Chemistry");
    Assert(Validate_Unique_Integers(Reactant1_Index, Product1_Index, Product2_Index),
            "Fission::Add_To_Chemistry - Duplicate Molecule Types");

//  First call the Add_To_Chemistry function for the base class (Interaction_Type)
//  to do the processing common to all Interaction_Types

    Interaction_Type*   Interaction_Type__Ptr;
    Interaction_Type__Ptr = this;
    Interaction_Type__Ptr->Add_To_Chemistry(Reactant1_Index, Product1_Index);

//  Link the Fission to Product 2 

    Molecule_Type__Product2_Ptr =  static_cast<Molecule_Type*>
        (Chemistry::Chemistry__Ptr->TList__Of_Molecule_Types_Ptr->Get_Item(Product2_Index));

//  Add the Catalytic Fission to the list of Interaction_Types for the
//  Product2 Molecule_Type 

    Molecule_Type__Product2_Ptr->List_Of_Int_Types_With_MolType_As_Input.Add(this);

//  Inform the Reactor of the new Interaction_Type

//    Reactor::Get_Reactor__Ptr()->Notify_New_Int_Type(this);  
    IntType_Notify_API::Notify_New_IntType(this);  
}
//----------------------------------------------------------------------------------------------------------

double Fission::Calculate_Reactor_Interaction_Rate
                 (const Calculate_Reactor_Interaction_Rates_Args* const Args) const
       
//  Calculate the Interaction Rate for this Interaction_Type using the Numbers of Molecules
//  of the Reactant and Products in the Reactor and the forward and reverese Reaction Rate 
//  constants for the Fission.

//  For the forward reaction, the Interaction Rate is proportional to the Number of Reactant Molecules

//  The Reverse Interaction Rate is calculated in the same way as the forward rate for a Construction.

//  The overall rate is either the Forward Rate minus the Reverse Rate, or zero if the Reverse Rate is 
//  greater than the Reverse Rate

{

//  Reactant2_Num_Mols argument is only set in the case of Interaction_Types with a Reactant2.
//  Check it was set correctly

    Assert(Args->Reactant2_Num_Mols < 0,
            "Fission::Calculate_Reactor_Interaction_Rate - Invalid Reactant2_Num_Mols Argument");

//  Product2_Num_Mols argument is only set in the case of Interaction_Types with a Product2.
//  Check it was set correctly

    Assert(Args->Product2_Num_Mols >= 0,
            "Fission::Calculate_Reactor_Interaction_Rate - Invalid Product2_Num_Mols Argument");


//  Convert arguments to double to ensure no integer overflow

    double R1_Num_Mols = static_cast<double> (Args->Reactant1_Num_Mols);
    double P1_Num_Mols = static_cast<double> (Args->Product1_Num_Mols);
    double P2_Num_Mols = static_cast<double> (Args->Product2_Num_Mols);

//  Make sure we don't have too many Molecules of either Reactant (forward reaction)
//  or of the Product (reverse reaction)

    Assert(R1_Num_Mols <= 100000000, 
        "Fission::Calculate_Reactor_Interaction_Rate: Number of Molecules of Reactant 1 exceeds 100000000.");
    Assert(P1_Num_Mols <= 100000000, 
        "Fission::Calculate_Reactor_Interaction_Rate: Number of Molecules of Product 1 exceeds 100000000.");
    Assert(P2_Num_Mols <= 100000000, 
        "Fission::Calculate_Reactor_Interaction_Rate: Number of Molecules of Product 2 exceeds 100000000.");
//  Reactant2_Num_Mols shoud be -1 in the case of a Fission. Check this is working.
    Assert(Args->Reactant2_Num_Mols  == -1,
            "Fission::Calculate_Reactor_Interaction_Rate - Invalid Argument");

//  Now calculate the Interaction Rate

    double Forward_Rate = Forward_Rate_Constant * R1_Num_Mols;
    double Reverse_Rate = (Reverse_Rate_Constant * (P1_Num_Mols * P2_Num_Mols))/ Args->Reactor_Volume;

//  ================================================
    return (max((Forward_Rate - Reverse_Rate),0.0));
//  ================================================
}
//----------------------------------------------------------------------------------------------------------

bool Fission::Check_If_Fission_Exists(int Reactant1_Index,
                                      int Product1_Index,
                                      int Product2_Index)

//  Check whether a Fission with the specified characteristics
//  already exists in the Chemistry
//  Do this by searching the list of Interaction_Types for Reactant 1.

{
    Molecule_Type*  Molecule_Type__Reactant1_Ptr;
    Interaction_Type*   Interaction_Type__Ptr;

    int         Interaction_Type_Identifier;
    Interaction_Type_Enum  Interaction_Type_Enumr;
    int         Reactant1_Identifier;
    int         Reactant2_Identifier;
    int         Product1_Identifier;
    int         Product2_Identifier;
    int         Catalyst_Identifier;

    Molecule_Type__Reactant1_Ptr
        = (Molecule_Type*) Chemistry::Chemistry__Ptr
            ->TList__Of_Molecule_Types_Ptr
                ->Get_Item(Reactant1_Index);

    for (int Continue_Flag = Molecule_Type__Reactant1_Ptr->List_Of_Int_Types_With_MolType_As_Input.Iter_To_First();
         Continue_Flag == true;
         Continue_Flag = Molecule_Type__Reactant1_Ptr->List_Of_Int_Types_With_MolType_As_Input.Iter_To_Next())
    BEGIN

        Interaction_Type__Ptr =
            Molecule_Type__Reactant1_Ptr->List_Of_Int_Types_With_MolType_As_Input.Get_Current_Item();

        Interaction_Type__Ptr->Get_Int_Type_Characteristics

                            (Interaction_Type_Identifier,
                             Interaction_Type_Enumr,
                             Reactant1_Identifier,

                             Reactant2_Identifier,
                             Product1_Identifier,
                             Product2_Identifier,
                             Catalyst_Identifier);

        if ((Interaction_Type_Enumr == Interaction_Type_Enumr__Fission)
                and
                (((Reactant1_Index == Reactant1_Identifier)
                    and
                (Product1_Index == Product1_Identifier)
                    and
                (Product2_Index == Product2_Identifier))
                            or
                ((Product2_Index == Product1_Identifier)
                    and
                (Product1_Index == Product2_Identifier)
                    and
                (Reactant1_Index == Reactant1_Identifier)))
           )
        THEN
            return(true);

        END
    END
    return(false);
}
//----------------------------------------------------------------------------------------------------------

void Fission::Determine_Rate_Constants()

//  Set the forward and reverse Rate Constants for the Fission.
//  Set them at random or according to Arrenhius rules depending on the argument

{   
    TRACE("Fission::Determine_Rate_Constants");
    if (Chemistry::Get_Rate_Constant_Model() == Chemistry::Arrenhius)
    THEN
        double Forward_Activation_Energy = Get_Forward_Activation_Energy();
        double Reverse_Activation_Energy = Get_Reverse_Activation_Energy();

//      ========================================================
        Forward_Rate_Constant = exp(-Forward_Activation_Energy);
        Reverse_Rate_Constant = exp(-Reverse_Activation_Energy);
//      ========================================================

    ELSEIF (Chemistry::Get_Rate_Constant_Model() == Chemistry::Random)
    THEN
        Forward_Rate_Constant = Random_Sequence::Get_Ptr(Random_Sequence::One)->Uniform(Max_Rand_Rate_Constant);
        Reverse_Rate_Constant = Random_Sequence::Get_Ptr(Random_Sequence::One)->Uniform(Max_Rand_Rate_Constant);
    ELSE
        Assert (true == false, "Fission::Determine_Rate_Constant() - Invalid Rate Constant Model");
    ENDIF
    Assert ((Forward_Rate_Constant >= 0) and (Reverse_Rate_Constant >= 0),  
             "Fission::Determine_Rate_Constant() - Invalid Rate_Constant");
}
//----------------------------------------------------------------------------------------------------------------


/*
Interaction_Type*  Fission::Copy_With_Variation(int Old_MolType_Index, int New_MolType_Index)

//  Make a modified copy of this Fission, such that if the existing Interaction_Type
//  has a reference to 'Old MolType', the new Fission is produced with the reference
//  replaced by a reference to 'New Mol Type'.


//  Make sure that all three Molecule_Types are different



{
    Molecule_Type*  Mol_Type_Reactant1_Ptr;
    Molecule_Type*  Mol_Type_Product1_Ptr;
    Molecule_Type*  Mol_Type_Product2_Ptr;

    if (Molecule_Type__Reactant1_Ptr->Get_Identifier() == Old_MolType_Index)
    THEN
        Mol_Type_Reactant1_Ptr = Chemistry::Get_Chemistry__Ptr()->Get_Molecule_Type_Ptr(New_MolType_Index);
    ELSE
        Mol_Type_Reactant1_Ptr = Molecule_Type__Reactant1_Ptr;
    ENDIF

    if (Molecule_Type__Product1_Ptr->Get_Identifier() == Old_MolType_Index)
    THEN
        Mol_Type_Product1_Ptr = Chemistry::Get_Chemistry__Ptr()->Get_Molecule_Type_Ptr(New_MolType_Index);
    ELSE
        Mol_Type_Product1_Ptr = Molecule_Type__Product1_Ptr;
    ENDIF

    if (Molecule_Type__Product2_Ptr->Get_Identifier() == Old_MolType_Index)
    THEN
        Mol_Type_Product2_Ptr = Chemistry::Get_Chemistry__Ptr()->Get_Molecule_Type_Ptr(New_MolType_Index);
    ELSE
        Mol_Type_Product2_Ptr = Molecule_Type__Product2_Ptr;
    ENDIF

//  Make sure that all Molecule Types are different. If OK, then create Constructio

    int  Reactant1_Index =   Mol_Type_Reactant1_Ptr->Get_Identifier();
    int  Product1_Index =    Mol_Type_Product1_Ptr->Get_Identifier();
    int  Product2_Index =    Mol_Type_Product2_Ptr->Get_Identifier();

    if (!Validate_Fission_Parameters(Reactant1_Index,
                                     Product1_Index,
                                     Product2_Index))
    THEN
        return  0;
    ELSE
        Interaction_Type*  New_Int_Ptr;
        CREATE_OBJECT(Fission, New_Int_Ptr = new  Fission(Mol_Type_Reactant1_Ptr,
                                                  Mol_Type_Product1_Ptr,
                                                  Mol_Type_Product2_Ptr,
						  Rate_Constant,       // Rate_Constant is unchanged
						  Activation_Energy));  
        return  New_Int_Ptr;
    ENDIF
} */
//----------------------------------------------------------------------------------------------------------
/*
void Fission::Draw(Network_Painter_Prot* const Network_Painter_Prot__Ptr,
                   double Interaction_Rate,
                   int Conn_Selection_State) const

//  Draw the Fission using the specified Network_Painter

{

//  Call the Network_Painter to display the Fission
//  Use the Molecule_Type Identifiers as the Node Indexes

    Network_Painter_Prot__Ptr->Draw_Fission(
                                           Molecule_Type__Reactant1_Ptr->Get_Identifier(),
                                           Molecule_Type__Product1_Ptr->Get_Identifier(),
                                           Molecule_Type__Product2_Ptr->Get_Identifier(),
                                           Interaction_Rate,
                                           Conn_Selection_State,
                                           true); // Draw for first time, so create a Network Display Item
                                                  // in case need to redraw;
}
*/

//----------------------------------------------------------------------------------------------------------

// bool Fission::Find_Candidate_Product_And_Energy(int Reactant_Index, 
//                                                 int Product1_Index,
//                                                 double& Fission_Actv_Cplx_Energy,
//                                                 int&  Product2_Index)
// 
// //  Find a candidate product and Activation Complex Energy for a Fission 
// //  based on the specified Reactant and Product
// 
// {
// //    int     Loop_Counter = 0;
// //    while(true) 
//     BEGIN
//  //       Assert(Loop_Counter++ < 1000, "Fission::Find_Candidate_Product_For_Fission - Cannot find suitable Product for Fission");
// 
//         Fission_Actv_Cplx_Energy = 
//             Random_Sequence::Get_Ptr(Random_Sequence::One)->
//                 Uniform(Interaction_Type::Get_Max_Activated_Complex_Energy());
// 
//         Product2_Index = Random_Sequence::Get_Ptr(Random_Sequence::One)->Uniform(Chemistry::Chemistry__Ptr->Number_Of_Molecule_Types);
//         if (Validate_Fission_Parameters(Reactant_Index, 
//                                         Product1_Index, 
//                                         Product2_Index,
//                                         Fission_Actv_Cplx_Energy) == true)
//         THEN
//             return true;
//         ELSE
//             return false;
//         ENDIF
//     END
// }
//----------------------------------------------------------------------------------------------------------------

Fission::Fission(int R_Id, int P1_Id, int P2_Id, double   Actv_Cplx_Energy)

//  Initialise Fission

{
    TRACE("Fission::Fission(int R_Id, int P1_Id, int P2_Id, double   Actv_Cplx_Energy");

//  Constructor - Base new Fission on specified Molecule_Types

    Molecule_Type__Reactant1_Ptr   = Chemistry::Get_MolType_Pt(R_Id);
    Molecule_Type__Product1_Ptr  = Chemistry::Get_MolType_Pt(P1_Id);
    Molecule_Type__Product2_Ptr  = Chemistry::Get_MolType_Pt(P2_Id);

    Assert (Molecule_Type__Reactant1_Ptr   != 0,  "Fission::Fission - MolType_R_Pt = 0");
    Assert (Molecule_Type__Product1_Ptr  != 0,  "Fission::Fission - MolType_P1_Pt  = 0");
    Assert (Molecule_Type__Product2_Ptr  != 0,  "Fission::Fission - MolType_P2_Pt  = 0");
    Assert (Actv_Cplx_Energy >= 0,"Fission::Fission - Invalid Actv_Cplx_Energy = 0");
    
    Assert (Molecule_Type__Reactant1_Ptr   != Molecule_Type__Product1_Ptr , "Fission::Fission - Reactant = Product 1");
    Assert (Molecule_Type__Product1_Ptr  != Molecule_Type__Product2_Ptr , "Fission::Fission - Product 1 = Product 2");
    Assert (Molecule_Type__Product2_Ptr  != Molecule_Type__Reactant1_Ptr  , "Fission::Fission - Product 2 = Reactant");

//  Activated_Complex_Energy must be higher than the Potential Energy of Reactant and of Products

    Assert (Actv_Cplx_Energy > (Molecule_Type__Product1_Ptr->Get_Potential_Energy() 
                                + Molecule_Type__Product2_Ptr->Get_Potential_Energy()),
            "Fission::Fission - Activated Complex Energy is less than Energy of Products");
    Assert (Actv_Cplx_Energy > (Molecule_Type__Reactant1_Ptr->Get_Potential_Energy()),
            "Fission::Fission - Activated Complex Energy is less than Energy of Reactant");
 
    Activated_Complex_Energy = Actv_Cplx_Energy;

    Determine_Rate_Constants();

//  Add this Fission to the Chemistry

    Add_To_Chemistry(Molecule_Type__Reactant1_Ptr->Get_Identifier(),
                     Molecule_Type__Product1_Ptr->Get_Identifier(),
                     Molecule_Type__Product2_Ptr->Get_Identifier());
}
//----------------------------------------------------------------------------------------------------------
//Fission::Fission()
//{

//  Setup unique characteristics for this Fission
//  and then add it to the Chemistry

//    int Reactant1_Index;
//    int Product1_Index;
//    int Product2_Index;

//    if (not Specify_Unique_Fission(Reactant1_Index,  
//				   Product1_Index, 
//				   Product2_Index, 
//				   Activated_Complex_Energy))
//    THEN
//      Couldn't find a new Fission

 //       throw Create_Error(string("Insufficient possibilities for new Fissions."));
//    ELSE

//      Add this Fission to the Chemistry

//        Add_To_Chemistry(Reactant1_Index, Product1_Index, Product2_Index);

 //       Determine_Rate_Constants();  // Do it after Add to Chemistry because MolType pointers are needed
//    ENDIF
//}
//---------------------------------------------------------------------------------------------------------

// Fission::Fission(Molecule_Type* Mol_Type__Reactant1_Ptr,
//                  Molecule_Type* Mol_Type__Product1_Ptr,
//                  Molecule_Type* Mol_Type__Product2_Ptr,
// 		         double         Actv_Cplx_Energy)
// 
// //  Constructor - Base new Fission on specified Molecule_Types
// 
// {
//     Assert (Mol_Type__Reactant1_Ptr!= 0, "Fission::Fission - Reactant1_Ptr = 0");
//     Assert (Mol_Type__Product1_Ptr!= 0, "Fission::Fission - Product1_Ptr = 0");
//     Assert (Mol_Type__Product2_Ptr!= 0, "Fission::Fission - Product2_Ptr = 0");
//     Assert (Actv_Cplx_Energy >= 0, "Fission::Fission - Invalid Activated_Complex_Energy = ");
// 
//     Assert (Mol_Type__Reactant1_Ptr not_eq Mol_Type__Product1_Ptr, "Fission::Fission - Reactant1 = Product1");
//     Assert (Mol_Type__Product1_Ptr not_eq Mol_Type__Product2_Ptr, "Fission::Fission - Product1 = Product2");
//     Assert (Mol_Type__Product2_Ptr not_eq  Mol_Type__Reactant1_Ptr, "Fission::Fission - Product2 = Reactant1");
// 
// //  Activated_Complex_Energy must be higher than the Potential Energy of Reactant and of Products
// 
//     Assert (Actv_Cplx_Energy > (Mol_Type__Product1_Ptr->Get_Potential_Energy() 
//                                 + Mol_Type__Product2_Ptr->Get_Potential_Energy()),
//             "Fission::Fission - Activated Complex Energy is less than Energy of Products");
//     Assert (Actv_Cplx_Energy > (Mol_Type__Reactant1_Ptr->Get_Potential_Energy()),
//             "Fission::Fission - Activated Complex Energy is less than Energy of Reactant");
//      
// 
//     Molecule_Type__Reactant1_Ptr = Mol_Type__Reactant1_Ptr;
//     Molecule_Type__Product1_Ptr = Mol_Type__Product1_Ptr;
//     Molecule_Type__Product2_Ptr = Mol_Type__Product2_Ptr;
// 
//     Activated_Complex_Energy = Actv_Cplx_Energy;
// 
//     Determine_Rate_Constants();
// 
// //  Add this Fission to the Chemistry
// 
//     Add_To_Chemistry(Molecule_Type__Reactant1_Ptr->Get_Identifier(),
//                      Molecule_Type__Product1_Ptr->Get_Identifier(),
//                      Molecule_Type__Product2_Ptr->Get_Identifier());
// }
//----------------------------------------------------------------------------------------------------------------

double Fission::Get_Forward_Activation_Energy() const

//  Return the Forward Activation Energy for the Fission

{
    TRACE("Fission::Get_Forward_Activation_Energy");

    double R1_PE = Get_Reactant1_MolType_Ptr()->Get_Potential_Energy();
    double Forward_Activation_Energy = Activated_Complex_Energy - R1_PE;
    return Forward_Activation_Energy;
}
//----------------------------------------------------------------------------------------------------------------

double  Fission::Get_Heat_Release()  const

//  Return the amount of heat released by one Fission.
//  For an endothermic Fission, the amount will be negative

{
    return (Get_Reactant1_MolType_Ptr()->Get_Potential_Energy() 
            - Get_Product1_MolType_Ptr()->Get_Potential_Energy()
            - Get_Product2_MolType_Ptr()->Get_Potential_Energy());
}
//----------------------------------------------------------------------------------------------------------
Interaction_Type_Enum  Fission::Get_Category() const

//  Return the category of Interaction_Type

{
    return(Interaction_Type_Enumr__Fission);
}
//----------------------------------------------------------------------------------------------------------------

string  Fission::Get_Category_As_String() const

//  Return the category of Interaction_Type

{
    return string("Fission");
}
//----------------------------------------------------------------------------------------------------------

void    Fission::Get_Int_Type_Characteristics
                                           (int& Interaction_Type_Identifier,
                                            Interaction_Type_Enum&  Interaction_Type_Enumr,
                                            int& Reactant1_Identifier,
                                            int& Reactant2_Identifier,
                                            int& Product1_Identifier,
                                            int& Product2_Identifier,
                                            int& Catalyst_Identifier) const

//  Return the characteristics of the Fission

{

    Reactant2_Identifier = 0;
    Catalyst_Identifier = 0;

    Interaction_Type_Identifier = Identifier;
    Interaction_Type_Enumr = Interaction_Type_Enumr__Fission;
    Reactant1_Identifier = Molecule_Type__Reactant1_Ptr->Get_Identifier();
    Product1_Identifier =  Molecule_Type__Product1_Ptr->Get_Identifier();
    Product2_Identifier =  Molecule_Type__Product2_Ptr->Get_Identifier();
}
//----------------------------------------------------------------------------------------------------------
void Fission::Get_Int_Type_Inputs_No_Dups(VCSList* TList__Of_Interaction_Inputs_Ptr) const

//  Put the Molecule_Type pointers for the Interaction_Type's inputs
//  (Reactants and Catalysts) in the TList. Don't include the
//  same Molecule_Type more than once.

{
    Assert(Molecule_Type__Reactant1_Ptr not_eq 0,
            "Fission::Get_Int_Type_Inputs_No_Dups - Invalid Reactant");
    TList__Of_Interaction_Inputs_Ptr->Clear();
    TList__Of_Interaction_Inputs_Ptr->Add(Molecule_Type__Reactant1_Ptr);
}
//----------------------------------------------------------------------------------------------------------

void Fission::Get_Int_Type_Products_With_Dups(VCSList* TList__Of_Int_Type_Products_Ptr) const

//  Put the Molecule_Type pointer for the Interaction_Type's Products
//  in the TList.

{

    TList__Of_Int_Type_Products_Ptr->Clear();
    TList__Of_Int_Type_Products_Ptr->Add(Molecule_Type__Product1_Ptr);
    TList__Of_Int_Type_Products_Ptr->Add(Molecule_Type__Product2_Ptr);
}
//----------------------------------------------------------------------------------------------------------

void Fission::Get_Int_Type_Reactants_With_Dups(VCSList* TList__Of_Int_Type_Reactants_Ptr) const

//  Put the Molecule_Type pointers for the Interaction_Type's Reactant
//  in the TList. 

{
    TList__Of_Int_Type_Reactants_Ptr->Clear();
    TList__Of_Int_Type_Reactants_Ptr->Add(Molecule_Type__Reactant1_Ptr);
}
//----------------------------------------------------------------------------------------------------------------

double Fission::Get_Reverse_Activation_Energy() const

//  Return the Reverse Activation Energy for the Fission

{
    TRACE("Fission::Get_Reverse_Activation_Energy");
    double P1_PE = Get_Product1_MolType_Ptr()->Get_Potential_Energy();
    double P2_PE = Get_Product2_MolType_Ptr()->Get_Potential_Energy();
    double Reverse_Activation_Energy = Activated_Complex_Energy - (P1_PE + P2_PE);
    return Reverse_Activation_Energy;
}
//----------------------------------------------------------------------------------------------------------


void    Fission::Remove_From_Chemistry()

//  Remove this Fission from the Chemistry by unlinking it from its Molecule_Types
//  and marking it as logically deleted.  Don't destroy the object; this will
//  enable other objects (eg in the Reactor) to continue to refer to it.

{

//  First call the Remove_From_Chemistry function for the base class (Interaction_Type)
//  to do the processing common to all Interaction_Types

    Interaction_Type::Remove_From_Chemistry();

//  Remove the Fission from the list of Interaction_Types for the
//  Product2 Molecule_Type, provided it was not removed earlier

    if (Molecule_Type__Product2_Ptr != Molecule_Type__Product1_Ptr)
    THEN
        Molecule_Type__Product2_Ptr->List_Of_Int_Types_With_MolType_As_Input.Remove(this);
    ENDIF
}
//----------------------------------------------------------------------------------------------------------

void Fission::Set_Max_Rand_Rate_Constant(double Max_Rate)

//  Set the maximum rate constant for Fission setup with the Random Rate Constant model

{
    Max_Rand_Rate_Constant = Max_Rate;
}
//----------------------------------------------------------------------------------------------------------------

bool Fission::Specify_Unique_Fission(bool R_Rnd, int& R_Id,   // Reactant
                                     bool P1_Rnd, int& P1_Id, // Product 1
                                     bool P2_Rnd, int& P2_Id, // Product 2
                                     bool ACE_Rnd_Fl, double& Actvtd_Cplx_Enrgy,
                                     string*& Fail_Tx_Pt)

//   Generate characteristics for a unique
//   Fission that does not already exist in the Chemistry
//   R_Rnd etc are set false if the value of the corresponding arguments specifies
//   a reactant / product that should not be changed. If the value is true then a
//   reactant / product must be chosen at random.

{
    TRACE("Fission::Specify_Unique_Fission");

    int Max_Attempts;
    if (R_Rnd or P1_Rnd or P2_Rnd or ACE_Rnd_Fl)
    THEN
        Max_Attempts = 1000;
    ELSE
        Max_Attempts = 1;  // No point in multiple attempts if everything is fixed
    ENDIF

    int  Loop_Counter = 0;
    do
    BEGIN

        ++Loop_Counter;

//      Keep trying until a Fisson that does not exist is found, or
//      return false if can't find one. Uniqueness is determined on the basis if
//      Inputs and Products only

        if (Loop_Counter > Max_Attempts)
        THEN
            if (Max_Attempts > 1)
            THEN
                Assert(Fail_Tx_Pt not_eq 0, "Fission::Specify_Unique_Fission - Invalid Fail_Tx_Pt");
                (*Fail_Tx_Pt) = string(" Insufficient feasible Fissions. Reason for failure of last attempt: ")
                              + (*Fail_Tx_Pt);
            ENDIF
            return false;
        ENDIF

        if (R_Rnd == true)
        THEN
            R_Id = Random_Sequence::Get_Ptr(Random_Sequence::One)
                        ->Uniform(Chemistry::Chemistry__Ptr->Number_Of_Molecule_Types);
        ELSE
//          Keep the specified Reactant
        ENDIF

        if (P1_Rnd == true)
        THEN
            P1_Id = Random_Sequence::Get_Ptr(Random_Sequence::One)
                        ->Uniform(Chemistry::Chemistry__Ptr->Number_Of_Molecule_Types);
        ELSE
//          Keep the specified Product 1
        ENDIF
        if (P2_Rnd == true)
        THEN
            P2_Id = Random_Sequence::Get_Ptr(Random_Sequence::One)
                        ->Uniform(Chemistry::Chemistry__Ptr->Number_Of_Molecule_Types);
        ELSE
//          Keep the specified Product 2
        ENDIF

//      Determine the Activated Complex Energy. It must be greater than energy of Reactants and of Product

        if (ACE_Rnd_Fl)
        THEN
            Actvtd_Cplx_Enrgy = Random_Sequence::Get_Ptr(Random_Sequence::One)
                                    ->Uniform(Get_Max_Activated_Complex_Energy()); 
        ELSE
//          Keep the specified value
        ENDIF
    END
    while ((Validate_Fission_Parameters(R_Id, 
                                        P1_Id, 
                                        P2_Id,
                                        Actvtd_Cplx_Enrgy,
                                        Loop_Counter == Max_Attempts,  // Report Fail Reason
                                        Fail_Tx_Pt)                    // only for last attempt
           == false));
    
    return true;   // and use Fail_Tx from validation
}
//----------------------------------------------------------------------------------------------------------------

bool Fission::Validate_Fission_Parameters(int R_Id,
                                          int P1_Id,
                                          int P2_Id,
                                          double Actv_Cplx_Energy,
                                          bool Rep_Fail_Reas_Fl,   // Whether to report a failure
                                          string*& Fail_Tx_Pt)

//  Fission Parameters are invalid if two Mol Types are equal, or
//  if another Fission with the same Molecule Types already exists
//  Also check that Mass of Products = Mass of Reactant.
//  And check that each Molecule_Type exists
//  In addition Activated Complex Energy must be higher than Reactant and higher than Products,
//  and the potential energy of the Reactant must be higher than that of the Products (so that
//  the forward rection is favoured).

{
    TRACE("Fission::Validate_Fission_Parameters");

    int Num_MolTypes = Chemistry::Get_Num_MolTypes();

    if((R_Id == P1_Id)
           ||    
       (P1_Id == P2_Id)
           ||
       (P2_Id == R_Id)
       )
    THEN
        if(Rep_Fail_Reas_Fl)
        THEN
            CREATE_OBJECT(string, Fail_Tx_Pt = new string(" Equal Molecule Types specified."));
        ENDIF
        return false;
        
    ELSEIF (Check_If_Fission_Exists(R_Id, P1_Id, P2_Id))
    THEN
        if(Rep_Fail_Reas_Fl)
        THEN
            CREATE_OBJECT(string, Fail_Tx_Pt = new string(" Fission already exists."));
        ENDIF
        return false;
    ELSEIF (R_Id >= Num_MolTypes
                    or
            R_Id < 0
                    or
            P1_Id >= Num_MolTypes
                    or
            P1_Id < 0
                    or
            P2_Id >= Num_MolTypes
                    or
            P2_Id < 0)
    THEN
        if(Rep_Fail_Reas_Fl)
        THEN
            CREATE_OBJECT(string, Fail_Tx_Pt = new string(" One of the specified Molecule Types is invalid or does not exist"));
        ENDIF
        return false;
    ELSE
        int R_Mass =  Chemistry::Get_MolType_Pt(R_Id)->Get_Mass();
        int P1_Mass =  Chemistry::Get_MolType_Pt(P1_Id)->Get_Mass();
        int P2_Mass =  Chemistry::Get_MolType_Pt(P2_Id)->Get_Mass();
        
        if (R_Mass not_eq (P1_Mass + P2_Mass))
        THEN
            if(Rep_Fail_Reas_Fl)
            THEN
                CREATE_OBJECT(string, Fail_Tx_Pt = new string(" Reactant mass is not the same as the mass of the Products."));
            ENDIF
            return false;
        ELSE   //  Validate the Activated Complex Energy and the Potential Energies
            double R_PE = Chemistry::Get_MolType_Pt(R_Id)->Get_Potential_Energy();
            double P1_PE = Chemistry::Get_MolType_Pt(P1_Id)->Get_Potential_Energy();
            double P2_PE = Chemistry::Get_MolType_Pt(P2_Id)->Get_Potential_Energy();
            if ((P1_PE + P2_PE) >= Actv_Cplx_Energy)
            THEN
                if(Rep_Fail_Reas_Fl)
                THEN
                    CREATE_OBJECT(string,  Fail_Tx_Pt = new string (" Activated Complex Energy is less than the sum of Product Potential Energies."));
                ENDIF
                return false;
            ELSEIF(R_PE >= Actv_Cplx_Energy)
            THEN
                if(Rep_Fail_Reas_Fl)
                THEN
                    CREATE_OBJECT(string, 
                        Fail_Tx_Pt = new string (" Activated Complex Energy is less than that of the Reactant."));
                ENDIF
                return false;
            ELSEIF(R_PE <= (P1_PE + P2_PE))
            THEN
                if(Rep_Fail_Reas_Fl)
                THEN
                    CREATE_OBJECT(string, 
                        Fail_Tx_Pt = new string (" Potential Energy of Products is greater than that of the Reactant."));
                ENDIF
                return false;
            ELSE
                return true;
            ENDIF
        ENDIF
    ENDIF
}                                                   
//----------------------------------------------------------------------------------------------------------

// bool Fission::Specify_Unique_Fission(bool R_Rnd,  int& R_Id,   // Reactant
//                                      bool P1_Rnd, int& P1_Id, // Product 1
//                                      bool P2_Rnd, int& P2_Id, // Product 2
//                                      bool ACE_Rnd_Fl, double& Actvtn_Enrgy,
//                                      string*& Fail_Tx_Pt)
// 
// //  Fission Parameters are invalid if two Mol Types are equal, or
// //  if another Fission with the same Molecule Types already exists
// //  Also validate mass conservation and Activated Complex Energy, and ensure
// //  that the Potential Energy of the Reactant is higher than that of the Products.
// 
// {
//     if((Reactant1_Index == Product2_Index)
//                 ||
//        (Product2_Index == Product1_Index)
//                 ||
//        (Product1_Index == Reactant1_Index)
//        )
//     THEN
//         return false;
// 
// 
//     ELSEIF (Check_If_Fission_Exists(Reactant1_Index, Product1_Index, Product2_Index))
//     THEN
//         return false;
//     ELSE
//     
// //        Chemistry* Chemistry_Ptr = Chemistry::Get_Chemistry__Ptr();
//         int Reactant1_Mass =  Chemistry::Get_MolType_Pt(Reactant1_Index)->Get_Mass();
//         int Product1_Mass =  Chemistry::Get_MolType_Pt(Product1_Index)->Get_Mass();
//         int Product2_Mass =  Chemistry::Get_MolType_Pt(Product2_Index)->Get_Mass();
//         
//         if ((Product1_Mass + Product2_Mass) not_eq Reactant1_Mass)
//         THEN
//             return false;
//         ELSE   //  Activated Complex Energy must be higher than that of Reactant and of Products
//          
//             double R1_PE = Chemistry::Get_MolType_Pt(Reactant1_Index)->Get_Potential_Energy();
//             double P1_PE = Chemistry::Get_MolType_Pt(Product1_Index)->Get_Potential_Energy();
//             double P2_PE = Chemistry::Get_MolType_Pt(Product2_Index)->Get_Potential_Energy();
// 
//             if (((P1_PE + P2_PE) >= Actv_Cplx_Energy)
//                         or
//                 (R1_PE >= Actv_Cplx_Energy)
//                         or
//                 ((P1_PE + P2_PE) >= R1_PE))
//             THEN
//                 return false;
//             ELSE
//                 return true;
//             ENDIF
//         ENDIF
//     ENDIF
// }
//----------------------------------------------------------------------------------------------------------


void    Interaction_Type::Add_To_Chemistry(int Reactant1_Index,
                                           int Product1_Index)

//  Base Class common processing to add an Interaction_Type to the Chemistry

//  and link the Interaction Type to its Input Molecule Types
//  and notify the Reactor of the new Interaction_Type

{

    Molecule_Type__Reactant1_Ptr =
        (Molecule_Type*) Chemistry::Get_MolType_Pt(Reactant1_Index);

    Assert(Molecule_Type__Reactant1_Ptr != 0, "Interaction_Type::Add_To_Chemistry - Molecule_Type__Reactant1_Ptr = 0");

//  Add the Interaction_Type to the list of Interaction_Types for the Reactant1 Molecule_Type


    Molecule_Type__Reactant1_Ptr->List_Of_Int_Types_With_MolType_As_Input.Add(this);

    Molecule_Type__Product1_Ptr =
        (Molecule_Type*) Chemistry::Get_MolType_Pt(Product1_Index);

//  Add the Interaction_Type to the list of Interaction_Types for the
//  Product1 Molecule_Type

    Molecule_Type__Product1_Ptr->List_Of_Int_Types_With_MolType_As_Product.Add(this);

//  Add to the list of Interaction_Types for the Chemistry

    Chemistry::Get_Chemistry__Ptr()->Add_Interaction_Type(this);

//  Inform the Reactor of the new Interaction_Type   ***** REMOVED - DO THIS IN THE DEVIVED CLASS
//                                                         WE CAN ONLY NOTIFY THE REACTOR AFTER THE DERIVED
//                                                         CLASS PROCESSING
//    Reactor::Get_Reactor__Ptr()->Notify_New_Int_Type(this);  
}
//----------------------------------------------------------------------------------------------------------
double Interaction_Type::Get_Activated_Complex_Energy() const

//  Return the Activated Complex Energy for the Interaction_Type

{
    return(Activated_Complex_Energy);
}
//----------------------------------------------------------------------------------------------------------

int Interaction_Type::Get_Identifier() const

//  Return the identifier for the Interaction_Type

{
    return(Identifier);
}
//----------------------------------------------------------------------------------------------------------------

Realised_Interaction_Type*  Interaction_Type::Get_Realised_Int_Type_Ptr()

//  Returns the pointer to the Realised Interaction Type (its zero if ther is not one)

{
    return Realised_Interaction_Type_Ptr;
}
//----------------------------------------------------------------------------------------------------------------

Interaction_Type::Interaction_Type()

//  Constructor for Interaction_Type

{
    Identifier = Next_Interaction_Type_Identifier;
    Next_Interaction_Type_Identifier++;
    Realisable_Interaction_Type__Ptr = 0;  // No R Int Type initially
    Realised_Interaction_Type_Ptr = 0;     // No Realised_Interaction_Type initially
}
//----------------------------------------------------------------------------------------------------------------

bool Interaction_Type::Is_Realisable()

//  Return true if the InteractionType is Realisable

{
    if (Realisable_Interaction_Type__Ptr)
    THEN
        return true;
    ELSE
        return false;
    ENDIF
}
//----------------------------------------------------------------------------------------------------------
void    Interaction_Type::Link_To_R_Int_Type(Realisable_Interaction_Type*  R_Int_Type_Ptr)

//  Link this Interaction_Type to the specified Realisable_Interaction_Type

{
    Assert (Realisable_Interaction_Type__Ptr == 0,
            "Interaction_Type::Link_To_R_Int_Type - Attempt to link R_Int_Type to an Interaction_Type that already has an R_Int_Type"); 
    Realisable_Interaction_Type__Ptr = R_Int_Type_Ptr;
} 
//----------------------------------------------------------------------------------------------------------------

void  Interaction_Type::Link_To_Realised_Int_Type(Realised_Interaction_Type* Realised_Int_Type_Ptr)

//  Link this Interaction_Type to the specified Realised_Interaction_Type

{
    Assert (Realised_Interaction_Type_Ptr == 0,
            "Interaction_Type::Link_To_Realised_Int_Type - Attempt to link Realised_Interaction_Type to an Interaction_Type that already has a Realised_Interaction_Type"); 
    Realised_Interaction_Type_Ptr = Realised_Int_Type_Ptr;
}
//----------------------------------------------------------------------------------------------------------
void    Interaction_Type::Remove_From_Chemistry()

//  Base Class common processing to remove an Interaction_Type from the Chemistry,
//  unlink from its Molecule Types, and remove any Realisable_Interaction_Type from the Reactor 

{

//  Remove the Interaction_Type from the list of Interaction_Types for the Reactant1 Molecule_Type

    Molecule_Type__Reactant1_Ptr->List_Of_Int_Types_With_MolType_As_Input.Remove(this);

//  Remove the Interaction_Type from the list of Interaction_Types for the Product1 Molecule_Type

    Molecule_Type__Product1_Ptr->List_Of_Int_Types_With_MolType_As_Product.Remove(this);

//  Remove from the list of Interaction_Types for the Chemistry

    Chemistry::Get_Chemistry__Ptr()->Remove_Interaction_Type(this);

//  If the Interaction_Type has a Realisable_Interaction_Type, then
//  mark it Unrealisable and Unlink it

    if (Realisable_Interaction_Type__Ptr != 0)
    THEN
//        Reactor::Get_Reactor__Ptr()->Notify_Removed_Int_Type(this);
        IntType_Notify_API::Notify_Removed_IntType(this);  
    END

//  Mark the Interaction_Type as logically deleted
//  *****  NEED TO MAKE SURE THAT IT GETS PHYSICALLY DELETED.

//    Removed = true;    
}
//----------------------------------------------------------------------------------------------------------
void    Interaction_Type::Unlink_From_R_Int_Type(Realisable_Interaction_Type*  R_Int_Type_Ptr)

//  Unlink  this Interaction_Type from the specified Realisable_Interaction_Type

{
//    cout  << "Enter Interaction_Type::Unlink_From_R_Int_Type"  << endl;
//    cout << "R_Int_Type_Ptr->Get_Interaction_Type_Ptr()->Get_Identifier()";
//    cout << R_Int_Type_Ptr->Get_Interaction_Type_Ptr()->Get_Identifier() << endl;
        
    Assert (Realisable_Interaction_Type__Ptr == R_Int_Type_Ptr,
            "Interaction_Type::Unlink_From_R_Int_Type - Attempt to unlink R_Int_Type from an Interaction_Type to which it is not linked"); 
    Realisable_Interaction_Type__Ptr = 0;
//    cout << "Interaction_Type::Unlink_From_R_Int_Type - Realisable_Interaction_Type__Ptr set to 0 for Int Type = " << this << endl;
 //   cout  << "Leave Interaction_Type::Unlink_From_R_Int_Type"  << endl;
}
//----------------------------------------------------------------------------------------------------------

void    Transformation::Add_To_Chemistry(int Reactant1_Index, int Product1_Index)

//  Add the Transformation to the Chemistry and link to Molecule_Types

{
    Assert(Validate_Unique_Integers(Reactant1_Index, Product1_Index),
            "Transformation::Add_To_Chemistry - Duplicate Molecule Types");

//  Call the Add_To_Chemistry function for the base class (Interaction_Type)
//  to do the processing common to all Interaction_Types

    Interaction_Type*   Interaction_Type__Ptr = this;
    Interaction_Type__Ptr->Add_To_Chemistry(Reactant1_Index, Product1_Index);

//  Inform the Reactor of the new Interaction_Type

//    Reactor::Get_Reactor__Ptr()->Notify_New_Int_Type(this);  
    IntType_Notify_API::Notify_New_IntType(this);  
}
//----------------------------------------------------------------------------------------------------------

 double Transformation::Calculate_Reactor_Interaction_Rate
    (const Calculate_Reactor_Interaction_Rates_Args* const Args) const


//  Calculate the Interaction Rate for this Interaction_Type using the Numbers of Molecules
//  of the Reactant and Products in the Reactor and the forward and reverese Reaction Rate 
//  constants for the Fission.

//  For the forward reaction, the Interaction Rate is proportional to the Number of Reactant Molecules
//  For the reverse reaction, the Interaction Rate is proportional to the Number of Product Molecules

//  The overall rate is either the Forward Rate minus the Reverse Rate, or zero if the Reverse Rate is 
//  greater than the Reverse Rate

{

//  Reactant2_Num_Mols argument is only set in the case of Interaction_Types with a Reactant2.
//  Check it was set correctly

    Assert(Args->Reactant2_Num_Mols == -1,
            "Transformation::Calculate_Reactor_Interaction_Rate - Invalid Reactant2_Num_Mols Argument");

//  Product2_Num_Mols argument is only set in the case of Interaction_Types with a Product2.
//  Check it was set correctly

    Assert(Args->Product2_Num_Mols == -1,
            "Transformation::Calculate_Reactor_Interaction_Rate - Invalid Product2_Num_Mols Argument");

//  Convert arguments to double to ensure no integer overflow

    double R_Num_Mols = static_cast<double> (Args->Reactant1_Num_Mols);
    double P_Num_Mols = static_cast<double> (Args->Product1_Num_Mols);

//  Make sure we don't have too many Molecules of either Reactant (forward reaction)
//  or of the Product (reverse reaction)

    Assert(R_Num_Mols <= 100000000, 
        "Transformation::Calculate_Reactor_Interaction_Rate: Number of Molecules of Reactant exceeds 100000000.");
    Assert(P_Num_Mols <= 100000000, 
        "Transformation::Calculate_Reactor_Interaction_Rate: Number of Molecules of Product exceeds 100000000.");

//  Now calculate the Interaction Rate

    double Forward_Rate = Forward_Rate_Constant * R_Num_Mols;
    double Reverse_Rate = Reverse_Rate_Constant * P_Num_Mols;

//  ================================================
    return (max((Forward_Rate - Reverse_Rate),0.0));
//  ================================================
}
//----------------------------------------------------------------------------------------------------------

bool Transformation::Check_If_Transformation_Exists(int Reactant1_Index, int Product1_Index)

//  Check whether a Transformation with the specified characteristics
//  already exists in the Chemistry
//  Do this by searching the list of Interaction_Types for Reactant 1.

{
    Molecule_Type*  Molecule_Type__Reactant1_Ptr;
    Interaction_Type*   Interaction_Type__Ptr;
 
    int         Interaction_Type_Identifier;
    Interaction_Type_Enum  Interaction_Type_Enumr;
    int         Reactant1_Identifier;
    int         Reactant2_Identifier;
    int         Product1_Identifier;
    int         Product2_Identifier;
    int         Catalyst_Identifier;

    Molecule_Type__Reactant1_Ptr
        = static_cast <Molecule_Type*>
            (Chemistry::Chemistry__Ptr->TList__Of_Molecule_Types_Ptr->Get_Item(Reactant1_Index));

    for (int Continue_Flag = Molecule_Type__Reactant1_Ptr->List_Of_Int_Types_With_MolType_As_Input.Iter_To_First();
         Continue_Flag == true;
         Continue_Flag = Molecule_Type__Reactant1_Ptr->List_Of_Int_Types_With_MolType_As_Input.Iter_To_Next())
    BEGIN

        Interaction_Type__Ptr =
            Molecule_Type__Reactant1_Ptr->List_Of_Int_Types_With_MolType_As_Input.Get_Current_Item();

        Interaction_Type__Ptr->Get_Int_Type_Characteristics
                            (Interaction_Type_Identifier,
                             Interaction_Type_Enumr,
                             Reactant1_Identifier,
                             Reactant2_Identifier,
                             Product1_Identifier,


                             Product2_Identifier,

                             Catalyst_Identifier);


        if ((Interaction_Type_Enumr == Interaction_Type_Enumr__Transfm)
                and
            (Reactant1_Index == Reactant1_Identifier)
                and
            (Product1_Index == Product1_Identifier)
           )
        THEN

            return(true);
        END
    END

    return(false);
}
//----------------------------------------------------------------------------------------------------------------
void Transformation::Determine_Rate_Constants()

//  Set the forward and reverse Rate Constants for the Transformation.
//  Set them at random or according to Arrenhius rules depending on the argument

{   
    TRACE("Transformation::Determine_Rate_Constants");
    if (Chemistry::Get_Rate_Constant_Model() == Chemistry::Arrenhius)
    THEN
        double Forward_Activation_Energy = Get_Forward_Activation_Energy();
        double Reverse_Activation_Energy = Get_Reverse_Activation_Energy();
//      ========================================================
        Forward_Rate_Constant = exp(-Forward_Activation_Energy);
        Reverse_Rate_Constant = exp(-Reverse_Activation_Energy);
//      ========================================================

    ELSEIF (Chemistry::Get_Rate_Constant_Model() == Chemistry::Random)
    THEN
        Forward_Rate_Constant = Random_Sequence::Get_Ptr(Random_Sequence::One)->Uniform(Max_Rand_Rate_Constant);
        Reverse_Rate_Constant = Random_Sequence::Get_Ptr(Random_Sequence::One)->Uniform(Max_Rand_Rate_Constant);
    ELSE
        Assert (false, "Transformation::Determine_Rate_Constants() - Invalid Rate Constant Model");
    ENDIF
    Assert ((Forward_Rate_Constant >= 0) and (Reverse_Rate_Constant >= 0), 
           "Transformation::Determine_Rate_Constant() - Invalid Rate_Constant");
}
//----------------------------------------------------------------------------------------------------------
/*Interaction_Type*  Transformation::Copy_With_Variation(int Old_MolType_Index, int New_MolType_Index)

//  Make a modified copy of this Transformation, such that if the existing Interaction_Type
//  has a reference to 'Old MolType', the new Transformation is produced with the reference
//  replaced by a reference to 'New Mol Type'.
//  Make sure that bothMolecule_Types are different


{
    Molecule_Type*  Mol_Type_Reactant1_Ptr;
    Molecule_Type*  Mol_Type_Product1_Ptr;

    if (Molecule_Type__Reactant1_Ptr->Get_Identifier() == Old_MolType_Index)
    THEN
        Mol_Type_Reactant1_Ptr = Chemistry::Get_Chemistry__Ptr()->Get_Molecule_Type_Ptr(New_MolType_Index);
    ELSE
        Mol_Type_Reactant1_Ptr = Molecule_Type__Reactant1_Ptr;
    ENDIF

    if (Molecule_Type__Product1_Ptr->Get_Identifier() == Old_MolType_Index)
    THEN
        Mol_Type_Product1_Ptr = Chemistry::Get_Chemistry__Ptr()->Get_Molecule_Type_Ptr(New_MolType_Index);
    ELSE
        Mol_Type_Product1_Ptr = Molecule_Type__Product1_Ptr;
    ENDIF

//  Make sure that both Molecule Types are different. If OK, then create Trabsformation


    int  Reactant1_Index =   Mol_Type_Reactant1_Ptr->Get_Identifier();
    int  Product1_Index =   Mol_Type_Product1_Ptr->Get_Identifier();
 
    if (!Validate_Transformation_Parameters(Reactant1_Index,
                                            Product1_Index))
    THEN
        return  0;

    ELSE
        Interaction_Type*  New_Int_Ptr;

        CREATE_OBJECT(Transformation, New_Int_Ptr = new  Transformation(Mol_Type_Reactant1_Ptr, Mol_Type_Product1_Ptr, Rate_Constant));
        return  New_Int_Ptr;
    ENDIF   
} */
//----------------------------------------------------------------------------------------------------------------

double Transformation::Get_Activation_Energy() const

//  Return the Activation Energy for the Transformation

{
    double R1_PE = Get_Reactant1_MolType_Ptr()->Get_Potential_Energy();
    double Activation_Energy = Activated_Complex_Energy - R1_PE;
    return Activation_Energy;
}
//----------------------------------------------------------------------------------------------------------
/*
void Transformation::Draw(Network_Painter_Prot* const Network_Painter_Prot__Ptr,
                        double Interaction_Rate,
                        int Conn_Selection_State) const


//  Draw the Transformation using the specified Network_Painter

{

//  Call the Network_Painter to display the Transformation
//  Use the Molecule_Type Identifiers as the Node Indexes

    Network_Painter_Prot__Ptr->Draw_Transformation(
                                                Molecule_Type__Reactant1_Ptr->Get_Identifier(),
                                                Molecule_Type__Product1_Ptr->Get_Identifier(),
                                                Interaction_Rate,
                                                Conn_Selection_State,
                                                true); // Draw for first time, so create a Network Display Item
                                                       // in case need to redraw;
}
*/
//----------------------------------------------------------------------------------------------------------
Interaction_Type_Enum  Transformation::Get_Category() const

//  Return the category of Interaction_Type

{
    return(Interaction_Type_Enumr__Transfm);
}
//----------------------------------------------------------------------------------------------------------------

string Transformation::Get_Category_As_String() const

//  Return the category of Interaction_Type

{
    return string("Transformation");
}
//----------------------------------------------------------------------------------------------------------------

double  Transformation::Get_Heat_Release()  const

//  Return the amount of heat released by one Transformation.
//  For an endothermic Transformation, the amount will be negative

{
    return (Get_Reactant1_MolType_Ptr()->Get_Potential_Energy() 
            - Get_Product1_MolType_Ptr()->Get_Potential_Energy());
}
//----------------------------------------------------------------------------------------------------------
void    Transformation::Get_Int_Type_Characteristics
                                           (int& Interaction_Type_Identifier,
                                            Interaction_Type_Enum&  Interaction_Type_Enumr,
                                            int& Reactant1_Identifier,
                                            int& Reactant2_Identifier,
                                            int& Product1_Identifier,
                                            int& Product2_Identifier,
                                            int& Catalyst_Identifier) const

//  Return the characteristics of the Transformation

{
    Product2_Identifier = 0;
    Reactant2_Identifier = 0;

    Catalyst_Identifier = 0;

//    std::string  AnsiString__Construction = "Transformation";

    Interaction_Type_Identifier = Identifier;
    Interaction_Type_Enumr = Interaction_Type_Enumr__Transfm;
    Reactant1_Identifier = Molecule_Type__Reactant1_Ptr->Get_Identifier();
    Product1_Identifier =  Molecule_Type__Product1_Ptr->Get_Identifier();
}

//----------------------------------------------------------------------------------------------------------
void Transformation::Get_Int_Type_Inputs_No_Dups(VCSList* TList__Of_Interaction_Inputs_Ptr) const

//  Put the Molecule_Type pointers for the Interaction_Type's inputs
//  (Reactants and Catalysts) in the TList. Don't include the
//  same Molecule_Type more than once. 

{
    Assert(Molecule_Type__Reactant1_Ptr not_eq 0,
            "Transformation::Get_Int_Type_Inputs_No_Dups - Invalid Reactant");
    TList__Of_Interaction_Inputs_Ptr->Clear();
    TList__Of_Interaction_Inputs_Ptr->Add(Molecule_Type__Reactant1_Ptr);
}
//----------------------------------------------------------------------------------------------------------
void Transformation::Get_Int_Type_Products_With_Dups(VCSList* TList__Of_Int_Type_Products_Ptr) const

//  Put the Molecule_Type pointer for the Interaction_Type's Product
//  in the TList.

{
    TList__Of_Int_Type_Products_Ptr->Clear();

    TList__Of_Int_Type_Products_Ptr->Add(Molecule_Type__Product1_Ptr);
}
//----------------------------------------------------------------------------------------------------------
void Transformation::Get_Int_Type_Reactants_With_Dups(VCSList* TList__Of_Int_Type_Reactants_Ptr) const

//  Put the Molecule_Type pointers for the Interaction_Type's Reactants
//  in the TList. If the same Reactant occurs twive then include it in the list twice (not
//  applicable for a Transformation).

{
    TList__Of_Int_Type_Reactants_Ptr->Clear();
    TList__Of_Int_Type_Reactants_Ptr->Add(Molecule_Type__Reactant1_Ptr);
}
//----------------------------------------------------------------------------------------------------------

void   Transformation::Remove_From_Chemistry()

//  Remove this Transformation from the Chemistry by unlinking it from its Molecule_Types
//  and marking it as logically deleted.  Don't destroy the object; this will
//  enable other objects (eg in the Reactor) to continue to refer to it.


{


//  First call the Remove_From_Chemistry function for the base class (Interaction_Type)
//  to do the processing common to all Interaction_Types

    Interaction_Type::Remove_From_Chemistry();


//  Nothing else to do for a Transformation

}
//----------------------------------------------------------------------------------------------------------

bool Transformation::Specify_Unique_Transformation(bool R_Rand_Fl, int& Reactant_Index,
                                                   bool P_Rand_Fl, int& Product_Index,
                                                   bool ACE_Rnd_Fl, double& Actvtd_Cplex_Enrgy,
                                                   string*& Fail_Tx_Pt)

//   Generate characteristics for a unique
//   Transformation that does not already exist in the Chemistry
//   R_Rand_Fl etc are set false if the value of the corresponding arguments specifies
//   a reactant / product that should not be changed. If the value is true then a
//   reactant / product must be chosen at random.

{
    TRACE("Transformation::Specify_Unique_Transformation");

    int Max_Attempts;
    if (R_Rand_Fl or P_Rand_Fl or ACE_Rnd_Fl)
    THEN
        Max_Attempts = 1000;
    ELSE
        Max_Attempts = 1;  // No point in multiple attempts if everything is fixed
    ENDIF

    int     Loop_Counter = 0;
    do
    BEGIN

        Loop_Counter++;

//      Keep trying until a Transformation that does not exist is found, or
//      return false if can't find one. Uniqueness is determined on the basis if
//      Reactant and Product only (not Rate_Constant)

        if (Loop_Counter > Max_Attempts)
        THEN
            if (Max_Attempts > 1)
            THEN
                Assert(Fail_Tx_Pt not_eq 0, "Transformation::Specify_Unique_Transformation - Invalid Fail_Tx_Pt");
               (*Fail_Tx_Pt) = string(" Insufficient feasible Transformations. Reason for failure of last attempt: ")
                              + (*Fail_Tx_Pt);
            ENDIF
            return false;
        ENDIF
        if (R_Rand_Fl == true)
        THEN
           Reactant_Index = Random_Sequence::Get_Ptr(Random_Sequence::One)
                ->Uniform(Chemistry::Chemistry__Ptr->Number_Of_Molecule_Types);
        ELSE
//          Keep the specified Reactant
        ENDIF

        if (P_Rand_Fl == true)
        THEN
            Product_Index = Random_Sequence::Get_Ptr(Random_Sequence::One)
                ->Uniform(Chemistry::Chemistry__Ptr->Number_Of_Molecule_Types);
        ELSE
//          Keep the specified Product
        ENDIF

//      Determine the Activated Complex Energy. It must be greater than energy of Reactant and of Product

        if (ACE_Rnd_Fl)
        THEN
            Actvtd_Cplex_Enrgy = Random_Sequence::Get_Ptr(Random_Sequence::One)
                ->Uniform(Get_Max_Activated_Complex_Energy()); 
        ELSE
//          Keep the specified value
        ENDIF

    END
    while ((Validate_Transformation_Parameters(Reactant_Index, 
                                               Product_Index, 
                                               Actvtd_Cplex_Enrgy,
                                               Loop_Counter == Max_Attempts,  // Report Fail Reason
                                               Fail_Tx_Pt)                    // for last attempt only


           == false));
    return true;
}
//----------------------------------------------------------------------------------------------------------
// Transformation::Transformation()
// {
// 
// //  Setup unique characteristics for this Transformation
// //  and then add it to the Chemistry
// 
//     int Reactant1_Index;
//     int Product1_Index;
// 
//     if (not Specify_Unique_Transformation(Reactant1_Index, Product1_Index, Forward_Rate_Constant))
//     THEN
// //      Couldn't find a new Transformation
// 
//         throw Create_Error(string("Insufficient possibilities for new Transformations."));
//     ELSE
// 
// //      Add this Transformation to the Chemistry
// 
//         Add_To_Chemistry(Reactant1_Index,Product1_Index);
//     ENDIF
// }
//----------------------------------------------------------------------------------------------------------
Transformation::Transformation(int R_Id, int P_Id, double   Actv_Cplx_Energy)

{
    TRACE("Transformation::Transformation(int R_Id, int P_Id, double   Actv_Cplx_Energy)");

    Molecule_Type__Reactant1_Ptr  = Chemistry::Get_MolType_Pt(R_Id);
    Molecule_Type__Product1_Ptr  = Chemistry::Get_MolType_Pt(P_Id);

    Assert (Molecule_Type__Reactant1_Ptr != 0, "Transformation::Transformation - Reactant1_Ptr = 0");
    Assert (Molecule_Type__Product1_Ptr != 0, "Transformation::Transformation - Product1_Ptr = 0");
    Assert (Actv_Cplx_Energy >= 0, "Transformation::Transformation - Invalid Activated_Complex_Energy");
    
    Assert (Molecule_Type__Product1_Ptr not_eq  Molecule_Type__Reactant1_Ptr  , "Transformation::Transformation - Reactant = Product");

//  Activated_Complex_Energy must be higher than the Potential Energy of Reactant and of Product

    Assert (Actv_Cplx_Energy > Molecule_Type__Reactant1_Ptr ->Get_Potential_Energy(),
            "Transformation::Transformation - Activated Complex Energy is not greater than Energy of Reactant");
    Assert (Actv_Cplx_Energy > (Molecule_Type__Product1_Ptr ->Get_Potential_Energy()),
            "Transformation::Transformation - Activated Complex Energy is not greater than Energy of Product");
 
    Activated_Complex_Energy = Actv_Cplx_Energy;

    Determine_Rate_Constants();

//  Add this Construction to the Chemistry

    Add_To_Chemistry(Molecule_Type__Reactant1_Ptr->Get_Identifier(),
                     Molecule_Type__Product1_Ptr->Get_Identifier());
}
//----------------------------------------------------------------------------------------------------------

bool Transformation::Validate_Transformation_Parameters(int Reactant_Index,
                                                        int Product_Index,
                                                        double Actv_Cplx_Energy,
                                                        bool Rep_Fail_Reas_Fl,
                                                        string*& Fail_Tx_Pt)

//  Transformation Parameters are invalid if two Mol Types are equal, or
//  if another Transformation with the same Molecule Types already exists
//  Also check that Mass of Product = Mass of Reactant.
//  And check that each Molecule_Type exists
//  In addition Activated Complex Energy must be higher than Reactant and higher than Product,
//  and the potential energy of the Reactants must be higher than that of the Product (so that
//  the forward rection is favoured).

{
    TRACE("Transformation::Validate_Transformation_Parameters");

    int Num_MolTypes = Chemistry::Get_Num_MolTypes();

    if(Reactant_Index == Product_Index)
    THEN
        if(Rep_Fail_Reas_Fl)
        THEN
            CREATE_OBJECT(string, Fail_Tx_Pt = new string(" Equal Molecule Types specified."));
        ENDIF
        return false;
    ELSEIF (Check_If_Transformation_Exists(Reactant_Index, Product_Index))
    THEN
        if(Rep_Fail_Reas_Fl)
        THEN
            CREATE_OBJECT(string, Fail_Tx_Pt = new string(" Transformation already exists."));
        ENDIF
        return false;
    ELSEIF (Reactant_Index >= Num_MolTypes
                    or
            Reactant_Index < 0
                    or
            Product_Index >= Num_MolTypes
                    or
            Product_Index < 0)
    THEN
        if(Rep_Fail_Reas_Fl)
        THEN
            CREATE_OBJECT(string, Fail_Tx_Pt = new string(" One of the specified Molecule Types is invalid or does not exist"));
        ENDIF
        return false;
    ELSE
        int Reactant_Mass =  Chemistry::Get_MolType_Pt(Reactant_Index)->Get_Mass();
        int Product_Mass =  Chemistry::Get_MolType_Pt(Product_Index)->Get_Mass();
        
        if (Reactant_Mass not_eq Product_Mass)
        THEN
            if(Rep_Fail_Reas_Fl)
            THEN
                CREATE_OBJECT(string, Fail_Tx_Pt = new string(" Product mass is not the same as Reactant mass."));
            ENDIF
            return false;
        ELSE   //  Validate the Activated Complex Energy and the Potential Energies
            double R_PE = Chemistry::Get_MolType_Pt(Reactant_Index)->Get_Potential_Energy();
            double P_PE = Chemistry::Get_MolType_Pt(Product_Index)->Get_Potential_Energy();
            if (R_PE  >= Actv_Cplx_Energy)
            THEN
                if(Rep_Fail_Reas_Fl)
                THEN
                    CREATE_OBJECT(string, 
                        Fail_Tx_Pt = new string (" Activated Complex Energy is less than Reactant Potential Energy."));
                ENDIF
                return false;
            ELSEIF(P_PE >= Actv_Cplx_Energy)
            THEN
                if(Rep_Fail_Reas_Fl)
                THEN
                    CREATE_OBJECT(string, 
                        Fail_Tx_Pt = new string (" Activated Complex Energy is less than Product Potential Energy."));
                ENDIF
                return false;
            ELSEIF(P_PE >= R_PE)
            THEN
                if(Rep_Fail_Reas_Fl)
                THEN
                    CREATE_OBJECT(string, 
                        Fail_Tx_Pt = new string (" Potential Energy of Reactant is less than that of the Product."));
                ENDIF
                return false;
            ELSE
                return true;
            ENDIF
        ENDIF
    ENDIF
}                                                   
//----------------------------------------------------------------------------------------------------------
